"""
Broadworks OCI-P Interface Base Classes

Base classes used by the types, requests and responses as well as
other components like ElementInfo that are used by those.
"""
import re
from collections import namedtuple

from classforge import Class
from classforge import Field
from lxml import etree

from broadworks_ocip.exceptions import OCIErrorResponse


class ElementInfo(
    namedtuple(
        "ElementInfo",
        ["name", "xmlname", "type", "is_complex", "is_required", "is_table"],
    ),
):
    """
    ElementInfo - information on each element of a Broadsoft OCIType

    Used to describe the element when serialising to/from XML
    """

    pass


class OCIType(Class):
    """
    OCIType - Base type for all the OCI-P component classes
    """

    _DEFAULT_NSMAP = {None: "", "xsi": "http://www.w3.org/2001/XMLSchema-instance"}
    _DOCUMENT_NSMAP = {None: "C", "xsi": "http://www.w3.org/2001/XMLSchema-instance"}
    _ERROR_NSMAP = {
        "c": "C",
        None: "",
        "xsi": "http://www.w3.org/2001/XMLSchema-instance",
    }

    @property
    def _type(self):
        return self.__class__.__name__

    def _post_xml_decode(self):
        """
        Carry out any operations after the XML decode

        Intended for use by subclasses where they need to take actions immediately
        after they are created from an incoming XML document.
        """
        pass

    def _etree_components(self, name=None):
        """
        Build XML etree element tree for this OCIType

        :param name: The name or tag of the element - defaults to the ``_type``
        :type name: string
        :rtype: etree.Element()

        """
        if name is None:
            name = self._type
        element = etree.Element(name, nsmap=self._DEFAULT_NSMAP)
        return self._etree_sub_components(element)

    def _etree_sub_components(self, element):
        """
        Build XML etree subelements for the components within this OCIType

        :param element: The element that is the parent of the subelements
        :type element: etree.Element()
        :rtype: etree.Element()

        """
        for sub_element in self._ELEMENTS:
            value = getattr(self, sub_element.name)
            if value is None:
                if sub_element.is_required:
                    etree.SubElement(
                        element,
                        sub_element.xmlname,
                        {"{http://www.w3.org/2001/XMLSchema-instance}nil": "true"},
                        nsmap=self._DEFAULT_NSMAP,
                    )
            elif sub_element.is_complex:
                sub_element.append(value._etree_components(sub_element.xmlname))
            else:
                elem = etree.SubElement(
                    element,
                    sub_element.xmlname,
                    nsmap=self._DEFAULT_NSMAP,
                )
                if sub_element.type == bool:
                    elem.text = "true" if value else "false"
                elif sub_element.type == int:
                    elem.text = str(value)
                else:
                    elem.text = value
        return element

    @classmethod
    def _column_header_snake_case(cls, header):
        """
        Converts an XML name into a pythonic snake case name

        :param header: The name to convert
        :type header: string
        :rtype: string

        """
        return re.sub("[ _]+", r"_", header).lower()

    @classmethod
    def _decode_table(cls, element):
        """
        Decode a table (used in a OCIResponse) into a list of named tuples

        :param element:
        :rtype: list of named tuples

        """
        typename = element.tag
        results = []
        columns = [
            cls._column_header_snake_case(b.text)
            for b in element.iterfind("colHeading")
        ]
        type = namedtuple(typename, columns)
        for row in element.iterfind("row"):
            rowdata = [b.text for b in row.iterfind("col")]
            rowobj = type(*rowdata)
            results.append(rowobj)
        return results

    @classmethod
    def _build_from_etree(cls, element):
        """
        Create an OciType based instance from an XML etree element

        :param element: XML etree element
        :type element: etree.Element()
        :rtype: cls instance

        """
        initialiser = {}
        for elem in cls._ELEMENTS:
            node = element.find(elem.xmlname)
            if node is not None:
                if elem.is_table:
                    initialiser[elem.name] = cls._decode_table(node)
                elif elem.is_complex:
                    initialiser[elem.name] = elem.type._build_from_etree(node)
                else:
                    initialiser[elem.name] = elem.type(node.text)
            # else...
            # I am inclined to thow an error here - at least after checking if
            # the thing is require, but the class builder should do that so lets
            # let it do its thing
        # now have a dict with all the bits in it.
        # use that to build a new object
        return cls(**initialiser)


class OCICommand(OCIType):
    """
    OCICommand - base class for all OCI Command (Request/Response) types
    """

    _session = Field(type=str, default="00000000-1111-2222-3333-444444444444")

    def _build_xml(self):
        """
        Build an XML document of the current Command (Request/Response)

        :rtype: string containing XML document
        """
        # document root element
        root = etree.Element(
            "{C}BroadsoftDocument",
            {"protocol": "OCI"},
            nsmap=self._DOCUMENT_NSMAP,
        )
        #
        # add the session
        sess = etree.SubElement(root, "sessionId", nsmap=self._DEFAULT_NSMAP)
        sess.text = self._session
        #
        # and the command
        element = self._build_xml_command_element(root)
        self._etree_sub_components(element)  # attach parameters etc
        #
        # wrap a tree around it
        tree = etree.ElementTree(root)
        return etree.tostring(
            tree,
            xml_declaration=True,
            encoding="ISO-8859-1",
            # standalone=False,
            # pretty_print=True,
        )

    def _build_xml_command_element(self, root):
        """
        Build the XML etree of the main command element of the current Command
        Intended to be overridden in a subclass for the few elements that do things
        a little differently (for example errors).

        :rtype: etree.Element()
        """
        return etree.SubElement(
            root,
            "command",
            {"{http://www.w3.org/2001/XMLSchema-instance}type": self._type},
            nsmap=self._DEFAULT_NSMAP,
        )


class OCIRequest(OCICommand):
    """
    OCIRequest - base class for all OCI Command Request types
    """

    pass


class OCIResponse(OCICommand):
    """
    OCIResponse - base class for all OCI Command Response types
    """

    pass


class SuccessResponse(OCIResponse):
    """
    The SuccessResponse is concrete response sent whenever a transaction is successful
    and does not return any data.
    """

    _ELEMENTS = ()  # type: ignore # type: Tuple[Tuple]


class ErrorResponse(OCIResponse):
    """
    The ErrorResponse is concrete response sent whenever a transaction fails and does not return any data.
    """

    _ELEMENTS = (
        ElementInfo("error_code", "errorCode", int, False, False, False),
        ElementInfo("summary", "summary", str, False, True, False),
        ElementInfo("summary_english", "summaryEnglish", str, False, True, False),
        ElementInfo("detail", "detail", str, False, False, False),
        ElementInfo("type", "type", str, False, False, False),
    )
    error_code = Field(type=int, required=False)
    summary = Field(type=str, required=True)
    summary_english = Field(type=str, required=True)
    detail = Field(type=str, required=False)
    type = Field(type=str, required=False)

    def _post_xml_decode(self):
        """Raise an exception as this is an error"""
        raise OCIErrorResponse(
            object=self,
            message=f"{self.error_code}: {self.summary} - {self.detail}",
        )

    def _build_xml_command_element(self, root):
        return etree.SubElement(
            root,
            "command",
            {
                "type": "Error",
                "echo": "",
                "{http://www.w3.org/2001/XMLSchema-instance}type": "c:" + self._type,
            },
            nsmap=self._ERROR_NSMAP,
        )


# end



# This code is based on https://medium.com/@hmajid2301/pytest-with-background-thread-fixtures-f0dc34ee3c46
import logging
import select
import socket
import sys

from broadworks_ocip import BroadworksAPI

FORMATTER = logging.Formatter(
    "%(asctime)s — %(name)s — %(levelname)s — %(funcName)s:%(lineno)d — %(message)s",
)

BASIC_API_PARAMS = {
    "host": "localhost",
    "username": "username@example.com",
    "password": "password",
    "session": "00000000-1111-2222-3333-444444444444",
}


class FakeServer:
    def __init__(self, port=0):
        self.port = port
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.configure_logger()
        self.logger.debug("Instantiated FakeServer")

    def __enter__(self):
        self.socket.bind(("127.0.0.1", self.port))
        host, port = self.socket.getsockname()
        self.logger.debug(f"Listening on tcp://{host}:{port}")
        self.port = port
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        try:
            self.socket.shutdown(socket.SHUT_RDWR)
        except OSError:
            # this seems to happen...
            pass
        self.socket.close()
        self.logger.info(f"Disconnected socket port={self.port}")

    def listen_for_traffic(self):
        while True:
            self.socket.listen(5)
            connection, address = self.socket.accept()
            (host, port) = address
            self.logger.info(f"Connection from host={host} port={port}")
            api = BroadworksAPI(**BASIC_API_PARAMS, logger=self.logger)

            while True:
                content = b""
                while True:
                    readable, writable, exceptional = select.select(
                        [connection],
                        [],
                        [connection],
                        5,
                    )
                    if readable:  # there is only one thing in the set...
                        try:
                            content += connection.recv(4096)
                        except ConnectionResetError:
                            self.logger.debug("Connection got reset")
                            connection.close()
                            return
                        # look for the end of document marker (we ignore line ends)
                        splits = content.partition(b"</BroadsoftDocument>")
                        if len(splits[1]) > 0:
                            break
                    elif exceptional:
                        self.logger.debug("Connection likely terminated")
                        connection.close()
                        return
                    elif not readable and not writable and not exceptional:
                        self.logger.debug("Read timeout")
                        connection.close()
                        return
                self.logger.debug(f"RECV: {str(content)}")
                self.process_command(connection, content, api)

    def configure_logger(self):
        """ """
        logger = logging.getLogger("fake_server")
        logger.setLevel(logging.DEBUG)
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setLevel(logging.DEBUG)
        console_handler.setFormatter(FORMATTER)
        logger.addHandler(console_handler)
        self.logger = logger

    def process_command(self, connection, content, api):
        cmd = api.decode_xml(content)
        self.logger.debug(f"F<<<: {cmd._type}")
        response = None
        if cmd._type == "AuthenticationRequest":
            cls = api.get_command_class("AuthenticationResponse")
            response = cls(
                user_id=cmd.user_id,
                nonce="1234567890123",
                password_algorithm="MD5",
            )
        elif cmd._type == "LoginRequest14sp4":
            # hard coded password corresponding to the hard coded params at top
            if cmd.signed_password == "77e4a6de3a1e00be05e121cf0ebee860":
                domain = cmd.user_id.partition("@")[2]
                if domain == "":
                    domain = "example.org"
                cls = api.get_command_class("LoginResponse14sp4")
                response = cls(
                    login_type="System",
                    locale="en_GB",
                    encoding="ISO-8859",
                    is_enterprise=False,
                    user_domain=domain,
                )
            else:
                cls = api.get_command_class("ErrorResponse")
                response = cls(
                    summary="[Error 4962] Invalid password",
                    summary_english="[Error 4962] Invalid password",
                )
        elif cmd._type == "SystemSoftwareVersionGetRequest":
            cls = api.get_command_class("SystemSoftwareVersionGetResponse")
            response = cls(version="21sp1")
        elif cmd._type == "LogoutRequest":
            # This normally never gets sent because the other end drops the connection
            cls = api.get_command_class("SuccessResponse")
            response = cls()
        else:
            cls = api.get_command_class("ErrorResponse")
            response = cls(
                error_code=9999,
                summary="Server is a fake and a fraud",
                summary_english="This is not a real server and doesn't implement many things",
                detail="There might be more detail if this was a real server",
                type="abject_panic",
            )
        response._session = cmd._session
        self.logger.debug(f"Built response {response._type}")
        xml = response._build_xml()
        self.logger.debug(f"F>>>: {response._type}")
        connection.sendall(xml + b"\n")
        self.logger.debug(f"SEND: {str(xml)}")



"""
Broadworks OCI-P Interface API Class and code

Main API interface - this is basically the only consumer visible part
"""
import hashlib
import inspect
import logging
import select
import socket
import sys
import uuid

from classforge import Class
from classforge import Field
from lxml import etree

import broadworks_ocip.base
import broadworks_ocip.requests
import broadworks_ocip.responses
import broadworks_ocip.types
from broadworks_ocip.exceptions import OCIErrorTimeOut


FORMATTER = logging.Formatter(
    "%(asctime)s — %(name)s — %(levelname)s — %(funcName)s:%(lineno)d — %(message)s",
)


class BroadworksAPI(Class):
    """
    BroadworksAPI - A class encapsulating the Broadworks OCI-P API
    """

    #: string: hostname/ip to connect to
    host = Field(type=str, required=True, mutable=False)
    #: int: port number to connect to. Default 2208
    port = Field(type=int, default=2208, mutable=False)
    #: string: username to authenticate with
    username = Field(type=str, required=True, mutable=False)
    #: string: password to authenticate with
    password = Field(type=str, required=True, mutable=False)
    #: logger to use - set up internally by default
    logger = Field(type=object)
    _despatch_table = Field(type=dict)
    #: boolean: are we authenticated?
    authenticated = Field(type=bool, default=False)
    #: integer: connection timeout value (default 8)
    connect_timeout = Field(type=int, default=8)
    #: integer: command timeout value (default 30)
    command_timeout = Field(type=int, default=30)
    #: socket: connection socket - set up internally
    socket = Field(type=object, default=None)
    #: string: session id - set up internally, only set this for testing
    session = Field(type=str)

    def on_init(self):
        """
        Initialise the API object
        """
        if self.session is None:
            self.session = str(uuid.uuid4())
        if self.logger is None:
            self.configure_logger()
        self.build_despatch_table()
        self.authenticated = False

    def build_despatch_table(self):
        """
        Create a despatch table of commands and types used
        """
        self.logger.debug("Building Broadworks despatch table")
        despatch_table = {}
        # deal with all the main request/responses
        for module in (broadworks_ocip.responses, broadworks_ocip.requests):
            for name, data in inspect.getmembers(module, inspect.isclass):
                if name.startswith("__"):
                    continue
                try:
                    if data.__module__ in (
                        "broadworks_ocip.types",
                        "broadworks_ocip.requests",
                        "broadworks_ocip.responses",
                    ):
                        despatch_table[name] = data
                except AttributeError:
                    continue
        # deal with special cases in base
        for name, data in inspect.getmembers(broadworks_ocip.base, inspect.isclass):
            if name in ("SuccessResponse", "ErrorResponse"):
                despatch_table[name] = data
                despatch_table["c:" + name] = data  # namespace issues
        # we now have a despatch table...
        self._despatch_table = despatch_table
        self.logger.debug("Built Broadworks despatch table")

    def configure_logger(self):
        """
        Create and configure a logging object
        """
        logger = logging.getLogger("broadworks_api")
        logger.setLevel(logging.DEBUG)
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setLevel(logging.DEBUG)
        console_handler.setFormatter(FORMATTER)
        logger.addHandler(console_handler)
        self.logger = logger

    def get_command_class(self, command):
        """
        Given a name (Request/Response/Type) name, return a class object for it

        :param command: A single word name of a OCIType(),OCIRequest(),OCIResponse()
        :type command: str
        :rtype: class object
        :raises: KeyError
        """
        try:
            cls = self._despatch_table[command]
        except KeyError as e:
            self.logger.error(f"Unknown command requested - {command}")
            raise e
        return cls

    def get_command_object(self, command, **kwargs):
        """
        Build the OCICommand object instance for a command and parameter

        :param command: A single word name of a OCIType(),OCIRequest(),OCIResponse()
        :type command: str
        :param kwargs: The parameters for the command
        :rtype: Object instance
        """
        cls = self.get_command_class(command)
        cmd = cls(_session=self.session, **kwargs)
        return cmd

    def get_command_xml(self, command, **kwargs):
        """
        Build the XML for a command and parameter

        :param command: A single word name of a OCIType(),OCIRequest(),OCIResponse()
        :type command: str
        :param kwargs: The parameters for the command
        :rtype: XML string
        """
        cmd = self.get_command_object(command, **kwargs)
        return cmd._build_xml()

    def send_command(self, command, **kwargs):
        """
        Build the XML for a command and parameter and send it to the server

        :param command: A single word name of a OCIType(),OCIRequest(),OCIResponse()
        :type command: str
        :param kwargs: The parameters for the command

        """
        self.logger.info(f">>> {command}")
        xml = self.get_command_xml(command, **kwargs)
        self.logger.debug(f"SEND: {str(xml)}")
        self.socket.sendall(xml + b"\n")

    def receive_response(self):
        """
        Wait and receive response XML from server, and decode it

        :rtype: Object instance
        :raises: OCIErrorTimeOut
        """
        content = b""
        while True:
            readable, writable, exceptional = select.select(
                [self.socket],
                [],
                [],
                self.command_timeout,
            )
            if readable:  # there is only one thing in the set...
                content += self.socket.recv(4096)
                # look for the end of document marker (we ignore line ends)
                splits = content.partition(b"</BroadsoftDocument>")
                if len(splits[1]) > 0:
                    break
            elif not readable and not writable and not exceptional:
                raise OCIErrorTimeOut(object=self, message="Read timeout")
        self.logger.debug(f"RECV: {str(content)}")
        return self.decode_xml(content)

    def decode_xml(self, xml):
        """
        Decode XML into an OCICommand based object instance

        :param xml:XML string
        :type command: str
        :rtype: Object instance
        """
        root = etree.fromstring(xml)
        if root.tag != "{C}BroadsoftDocument":
            raise ValueError
        self.logger.debug("Decoding BroadsoftDocument")
        for element in root:
            if element.tag == "command":
                command = element.get("{http://www.w3.org/2001/XMLSchema-instance}type")
                self.logger.debug(f"Decoding command {command}")
                cls = self._despatch_table[command]
                result = cls._build_from_etree(element)
                self.logger.info(f"<<< {result._type}")
                result._post_xml_decode()
                return result

    def connect(self):
        """
        Open the connection to the OCI-P server

        :raises: OSError
        :raises: socket.timeout
        """
        self.logger.debug(f"Attempting connection host={self.host} port={self.port}")
        try:
            address = (self.host, self.port)
            conn = socket.create_connection(
                address=address,
                timeout=self.connect_timeout,
            )
            self.socket = conn
            self.logger.info(f"Connected to host={self.host} port={self.port}")
        except OSError as e:
            self.logger.error("Connection failed")
            raise e
        except socket.timeout as e:
            self.logger.error("Connection timed out")
            raise e

    def authenticate(self):
        """
        Authenticate the connection to the OCI-P server

        :raises: OCIErrorResponse
        """
        self.send_command("AuthenticationRequest", user_id=self.username)
        resp = self.receive_response()
        authhash = hashlib.sha1(self.password.encode()).hexdigest().lower()
        signed_password = (
            hashlib.md5(":".join([resp.nonce, authhash]).encode()).hexdigest().lower()
        )
        self.send_command(
            "LoginRequest14sp4",
            user_id=self.username,
            signed_password=signed_password,
        )
        # if this fails to authenticate an ErrorResponse will be returned which forces
        # an exception to be raised
        resp = self.receive_response()
        # if authentication failed this line will never be executed
        self.authenticated = True

    def command(self, command, **kwargs):
        """
        Send a command and parameters to the server, receive and decode a response

        :param command: A single word name of a OCIType(),OCIRequest(),OCIResponse()
        :type command: str
        :param kwargs: The parameters for the command
        :rtype: Object instance
        :raises: OCIErrorResponse
        :raises: OCIErrorResponse
        """
        if not self.authenticated:
            self.connect()
            self.authenticate()
        self.send_command(command, **kwargs)
        return self.receive_response()

    def close(self):
        """
        Close the connection to the OCI-P server
        """
        if self.authenticated:
            self.logger.debug("Disconnect by logging out")
            self.send_command(
                "LogoutRequest",
                user_id=self.username,
                reason="Connection close",
            )
            self.authenticated = False
        if self.socket:
            self.socket.shutdown(socket.SHUT_RDWR)
            self.socket.close()
            self.logger.info(f"Disconnected from host={self.host} port={self.port}")
            self.socket = None

    def __del__(self):
        self.close()


# end



#!/usr/bin/env python
"""The setup script."""
from setuptools import find_packages
from setuptools import setup

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = ["classforge>=0.92", "lxml>=4.5.0"]

setup_requirements = [
    "pytest-runner",
]

test_requirements = [
    "pytest>=3",
]

setup(
    author="Nigel Metheringham",
    author_email="nigelm@cpan.org",
    python_requires=">=3.5",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="Interfaces to the OCI-P provisioning interface of a Broadworks softswitch",
    install_requires=requirements,
    license="BSD license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="broadworks_ocip",
    name="broadworks_ocip",
    packages=find_packages(include=["broadworks_ocip", "broadworks_ocip.*"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/nigelm/broadworks_ocip",
    version="0.1.1",
    zip_safe=False,
)



"""
Broadworks OCI-P Interface Exception Classes

Exception classes used by the API.
"""


class OCIError(Exception):
    """Base Exception raised by OCI operations.

    Attributes:
        message -- explanation of why it went bang
        object -- the thing that went bang
    """

    def __init__(self, message, object=None):
        self.message = message
        self.object = object

    def __str__(self):
        return f"{self.__class__.__name__}({self.message})"


class OCIErrorResponse(OCIError):
    pass


class OCIErrorTimeOut(OCIError):
    pass


# end



#!/usr/bin/env python
"""Tests for `broadworks_ocip` package."""
import pytest  # noqa: F401
from lxml import etree

from broadworks_ocip import BroadworksAPI

BASIC_API_PARAMS = {
    "host": "localhost",
    "username": "username@example.com",
    "password": "password",
    "session": "00000000-1111-2222-3333-444444444444",
}


def canon_xml(inxml):
    """
    we XML canonicalise/pretty print this to prevent whitespace/quote ordering
    issues and to make any diffs easier to read.
    Returned XML is as a decoded string - prettier for diffs
    """
    return etree.tostring(etree.fromstring(inxml), pretty_print=True).decode()


def compare_command_xml(wanted, command, **kwargs):
    """Create a Broadworks XML command framgment from the argumenta"""
    api = BroadworksAPI(**BASIC_API_PARAMS)
    generated = api.get_command_xml(command, **kwargs)
    assert canon_xml(generated) == canon_xml(wanted)


def test_authentication_request_xml():
    compare_command_xml(
        (
            b'<?xml version="1.0" encoding="ISO-8859-1"?>\n'
            b'<BroadsoftDocument protocol="OCI" xmlns="C" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">'
            b'<sessionId xmlns="">00000000-1111-2222-3333-444444444444</sessionId>'
            b'<command xmlns="" xsi:type="AuthenticationRequest">'
            b"<userId>username@example.com</userId></command></BroadsoftDocument>"
        ),
        "AuthenticationRequest",
        user_id="username@example.com",
    )


def test_error_response_xml():
    compare_command_xml(
        (
            b'<?xml version="1.0" encoding="ISO-8859-1"?>\n'
            b'<BroadsoftDocument protocol="OCI" xmlns="C" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">'
            b'<sessionId xmlns="">00000000-1111-2222-3333-444444444444</sessionId>'
            b'<command type="Error" echo="" xsi:type="c:ErrorResponse" xmlns:c="C" xmlns="">'
            b"<summary>[Error 4962] Invalid password</summary>"
            b"<summaryEnglish>[Error 4962] Invalid password</summaryEnglish>"
            b"</command></BroadsoftDocument>"
        ),
        "ErrorResponse",
        summary="[Error 4962] Invalid password",
        summary_english="[Error 4962] Invalid password",
    )


# end



#!/usr/bin/env python
# This code is based on https://medium.com/@hmajid2301/pytest-with-background-thread-fixtures-f0dc34ee3c46
"""Tests for `broadworks_ocip` package."""
import threading

import pytest  # noqa: F401

from broadworks_ocip import BroadworksAPI
from broadworks_ocip import OCIErrorResponse
from tests.fakeserver import FakeServer

SERVER_TCP_PORT = 2208
BASIC_API_PARAMS = {
    "host": "127.0.0.1",
    "port": SERVER_TCP_PORT,
    "username": "username@example.com",
    "password": "password",
    "session": "00000000-1111-2222-3333-444444444444",
    "connect_timeout": 3,
}


@pytest.fixture(autouse=True)
def dummy_tcp_server():
    tcp_server = FakeServer(port=SERVER_TCP_PORT)
    with tcp_server as example_server:
        thread = threading.Thread(target=example_server.listen_for_traffic)
        thread.daemon = True
        thread.start()
        yield example_server


def test_server_login():
    api = BroadworksAPI(**BASIC_API_PARAMS)
    assert api is not None
    assert api.__class__.__name__ == "BroadworksAPI"
    assert api.authenticated is False
    response = api.command("SystemSoftwareVersionGetRequest")
    assert api.authenticated is True
    assert response is not None
    assert response._type == "SystemSoftwareVersionGetResponse"


def test_server_login_fail():
    params = BASIC_API_PARAMS
    params["password"] = "wrong wrong wrong"
    api = BroadworksAPI(**params)
    assert api is not None
    assert api.__class__.__name__ == "BroadworksAPI"
    assert api.authenticated is False
    with pytest.raises(OCIErrorResponse) as excinfo:
        api.command("SystemSoftwareVersionGetRequest")
    assert "Invalid password" in str(excinfo.value)
    assert api.authenticated is False


# end
